"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const react = require("react");
const view = require("@codemirror/view");
function runTask(callback) {
  let task = callback;
  queueMicrotask(() => task == null ? void 0 : task());
  return () => task = null;
}
function createAsyncScheduler() {
  let cancelTask;
  return {
    request: (callback) => cancelTask = runTask(callback),
    cancel: () => cancelTask == null ? void 0 : cancelTask()
  };
}
function isFunction(value) {
  return typeof value === "function";
}
function createCodeMirror(config) {
  let prevState;
  let currentView = null;
  function createConfig() {
    return (isFunction(config) ? config : () => config)(prevState);
  }
  function createView(container) {
    return new view.EditorView({
      ...createConfig(),
      parent: container
    });
  }
  const viewChangeCallbacks = /* @__PURE__ */ new Set();
  function subscribeViewChange(callback) {
    viewChangeCallbacks.add(callback);
    return () => viewChangeCallbacks.delete(callback);
  }
  function publishViewChange() {
    viewChangeCallbacks.forEach((callback) => callback());
  }
  function setView(view2) {
    if (view2 === currentView) {
      return;
    }
    if (currentView) {
      prevState = currentView.state;
      currentView.destroy();
    }
    currentView = view2;
    publishViewChange();
  }
  const getView = () => currentView;
  const getServerView = () => null;
  const useView = () => {
    const view2 = react.useSyncExternalStore(subscribeViewChange, getView, getServerView);
    react.useDebugValue(view2);
    return view2;
  };
  const useViewEffect = (effect, deps) => {
    const view2 = useView();
    react.useEffect(() => {
      if (view2) {
        return effect(view2);
      }
    }, [view2, ...deps]);
  };
  const useViewDispatch = () => react.useCallback((...specs) => {
    const view2 = getView();
    if (!view2) {
      throw new TypeError("Cannot dispatch transaction without a view");
    }
    view2.dispatch(...specs);
  }, []);
  function createContainerRef() {
    let currentContainer = null;
    const scheduler = createAsyncScheduler();
    return Object.seal({
      get current() {
        return currentContainer;
      },
      set current(container) {
        if (container === currentContainer) {
          return;
        }
        currentContainer = container;
        scheduler.cancel();
        scheduler.request(() => {
          setView(null);
          setView(container && createView(container));
        });
      }
    });
  }
  let currentContainerRef;
  function getContainerRef() {
    return currentContainerRef != null ? currentContainerRef : currentContainerRef = createContainerRef();
  }
  const useContainerRef = () => {
    const containerRef = getContainerRef();
    react.useDebugValue(containerRef);
    return containerRef;
  };
  return {
    getView,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
function toPascalCase(str) {
  return str.split(/[-_]/).map((word) => word && word[0].toUpperCase() + word.slice(1)).join("");
}
function useSingleton(instance) {
  const instanceRef = react.useRef(null);
  if (instanceRef.current == null) {
    instanceRef.current = isFunction(instance) ? instance() : instance;
  }
  return instanceRef.current;
}
function createCodeMirrorWithContext(displayName) {
  const InternalCodeMirrorContext = react.createContext(null);
  const CodeMirrorProvider = ({ config, children }) => {
    const instance = useSingleton(() => createCodeMirror(config));
    return /* @__PURE__ */ react.createElement(
      InternalCodeMirrorContext.Provider,
      { value: instance },
      children
    );
  };
  if (displayName) {
    displayName = toPascalCase(displayName);
    CodeMirrorProvider.displayName = `${displayName}.Provider`;
    InternalCodeMirrorContext.displayName = `Internal${displayName}`;
  }
  const useCodeMirrorContext = () => {
    const instance = react.useContext(InternalCodeMirrorContext);
    if (!instance) {
      throw new Error(
        "could not find instance from CodeMirrorContext; please ensure the component is wrapped in a <Provider>"
      );
    }
    return instance;
  };
  const useGetView = () => {
    const { getView: getContextView } = useCodeMirrorContext();
    return getContextView;
  };
  const useView = () => {
    const { useView: useContextView } = useCodeMirrorContext();
    return useContextView();
  };
  const useViewEffect = (effect, deps) => {
    const { useViewEffect: useContextViewEffect } = useCodeMirrorContext();
    return useContextViewEffect(effect, deps);
  };
  const useViewDispatch = () => {
    const { useViewDispatch: useContextViewDispatch } = useCodeMirrorContext();
    return useContextViewDispatch();
  };
  const useContainerRef = () => {
    const { useContainerRef: useContextContainerRef } = useCodeMirrorContext();
    return useContextContainerRef();
  };
  return {
    Provider: CodeMirrorProvider,
    useContext: useCodeMirrorContext,
    useGetView,
    useView,
    useViewEffect,
    useViewDispatch,
    useContainerRef
  };
}
exports.createCodeMirror = createCodeMirror;
exports.createCodeMirrorWithContext = createCodeMirrorWithContext;
//# sourceMappingURL=index.cjs.map
